package com.agileworks.machine;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertSame;
import static org.junit.Assert.assertTrue;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

import org.junit.Test;

public class ControllerTest implements Panel, Cashier, Dispenser {
    private Controller controller;
    private List<String> messages = new ArrayList<String>();
    private CoffeeType servedCoffee;
    private int insertedAmount;
    private final int coffeePrice = 35;

    @Test
    public void serveSelectedCoffeeWhenUserInputsExactChange() {
        powerOn().displays("Please select a coffee type");
        select(CoffeeType.BLACK).displays("Please insert .35$");
        insert(AmericanCoins.THIRTY_FIVE_CENTS).displays("Please collect your delicious coffee");
        collectCoffee(CoffeeType.BLACK);
        displays("Please select a coffee type");
    }

    @Test
    public void returnChangeAfterServingSelectedCoffee() {
        powerOn().displays("Please select a coffee type");
        select(CoffeeType.BLACK).displays("Please insert .35$");
        insert(AmericanCoins.DOLLAR).displays("Please collect your change of .65$ and your delicious coffee");
        collectChange(AmericanCoins.SIXTY_FIVE_CENTS);
        collectCoffee(CoffeeType.BLACK);
        displays("Getting Ready...");
        displays("Please select a coffee type");
    }

    private void collectCoffee(final CoffeeType type) {
        assertSame(type, servedCoffee);
    }

    private void collectChange(final AmericanCoins change) {
        assertEquals(change.value(), insertedAmount - coffeePrice);
    }

    private ControllerTest insert(final AmericanCoins coins) {
        controller.accept(coins);
        insertedAmount = coins.value();
        return this;
    }

    private ControllerTest select(final CoffeeType type) {
        controller.selectedCoffee(type);
        return this;
    }

    private void displays(final String msg) {
        assertTrue("Message: " + msg + " not found\n Messages Recieved are:" + messages, messages.contains(msg));
    }

    private ControllerTest powerOn() {
        controller = new Controller(this, this, this);
        return this;
    }

    @Override
    public void display(final String msg) {
        messages.add(msg);
    }

    @Override
    public String priceFor(final CoffeeType type) {
        return "." + coffeePrice + "$";
    }

    @Override
    public String returnChangeFor(final CoffeeType type, final AmericanCoins coins) {
        return "." + (coins.value() - coffeePrice) + "$";
    }

    @Override
    public void serve(final CoffeeType type) {
        servedCoffee = type;
    }

    @Override
    public List<AmericanCoins> unavailableCoins() {
        return Collections.emptyList();
    }

    @Override
    public List<CoffeeType> unavailableCoffeTypes() {
        return Collections.emptyList();
    }

    @Override
    public void unavailable(final List<CoffeeType> unavailableCoffees) {
        assertTrue(unavailableCoffees.isEmpty());
    }
}
